@@ -144,8 +144,8 @@ class SteadystateProblem {
      * @brief Computes the weighted root mean square of xdot
      * the weights are computed according to x:
      * w_i = 1 / ( rtol * x_i + atol )
-     * @param x current state
-     * @param xdot current rhs
+     * @param x current state (sx[ip] for sensitivities)
+     * @param xdot current rhs (sxdot[ip] for sensitivities)
      * @param atol absolute tolerance
      * @param rtol relative tolerance
      * @param ewt error weight vector
@@ -248,7 +248,6 @@ class SteadystateProblem {
         return x_;
     };
 
-
     /**
      * @brief Returns state sensitivity at steadystate
      * @return sx
@@ -321,6 +320,12 @@ class SteadystateProblem {
      */
     void getAdjointUpdates(Model &model, const ExpData &edata);
 
+    /**
+     * @brief Return the adjoint state
+     * @return xB adjoint state
+     */
+    AmiVector const& getAdjointState() const { return xB_; }
+
     /**
      * @brief Accessor for xQB
      * @return xQB
@@ -372,7 +377,7 @@ class SteadystateProblem {
     AmiVector xQ_;
     /** quadrature state vector */
     AmiVector xQB_;
-    /** quadrature state vector */
+    /** time-derivative of quadrature state vector */
     AmiVector xQBdot_;
 
     /** maximum number of steps for Newton solver for allocating numlinsteps */
